<?php

namespace Drupal\gitinfo\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Class GitinfoSettingsForm.
 */
class GitinfoSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'gitinfo.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'gitinfo_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('gitinfo.settings');
    $status_report = Link::fromTextAndUrl($this->t('Status Report'), Url::fromRoute('system.status'));
    $status_report = $status_report->toRenderable();
    $form['link'] = [
      '#type' => 'item',
      '#description' => $this->t("Configuration for the GIT Info section in the @status_report.", ['@status_report' => render($status_report)]),
    ];
    $form['connect'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Connect with remote repos'),
      '#description' => $this->t("<strong>Slows down report generation</strong> but it tells if your working directory is behind compared with the remote repo. In other words, if a 'git pull' is needed. Furthermore, it shows if your Git version is up to date."),
      '#default_value' => $config->get('connect'),
    ];
    $explain_title = NULL;
    $use_global = Link::fromTextAndUrl($this->t('use a global variable in the <em>settings.php</em>'), Url::fromUri('https://www.drupal.org/docs/8/modules/git-info-report/how-to-define-directories-outside-the-project-root#s-configuration-using-a-global'));
    $use_global = $use_global->toRenderable();
    $use_global['#attributes'] = ['title' => $this->t('GIT Info Report - How to define directories outside the project root | Drupal 8 guide on Drupal.org'), 'target' => '_blank'];
    $description = $this->t('For example &#039;/var/www/html&#039; to monitor all projects in that folder. One directory per line. The <strong>absolute path</strong> from system root must be used, starting with a slash ( / ).<br />Alternatively, @use_global. Any projects that contain folders that are under Git version control can be added. Also those running in a Virtual Machine or non Drupal projects.', ['@use_global' => render($use_global)]);
    $disabled = FALSE;
    global $_gitinfo_extra_folders_;
    if (isset($_gitinfo_extra_folders_)) {
      $explain_title = $this->t("(<strong>FIELD INPUT DISABLED</strong>)");
      $description = $this->t("Overridden by the global variable '\$_gitinfo_extra_folders_' in the 'settings.php' file with the values (comma separated):<br /><em><strong>$_gitinfo_extra_folders_</strong></em>");
      $disabled = TRUE;
    }
    $form['extra_folders'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Extra folders to monitor outside the current project root @explain_title', ['@explain_title' => $explain_title]),
      '#description' => $description,
      '#default_value' => $config->get('extra_folders'),
      '#disabled' => $disabled,
    ];
    $form['aggregate_folders'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Aggregate items in ignored folders to show only an item count per directory'),
      '#description' => $this->t("To save vertical space in the report. One directory path per line. The <strong>relative path</strong> from the project root must be used. Read the project's <em>.gitignore</em> file for inspiration or just see which folders result in long lists.<br />Leave empty to reset to the default."),
      '#default_value' => $config->get('aggregate_folders'),
    ];
    $form['number_commit_messages'] = [
      '#type' => 'number',
      '#title' => $this->t('Number of latest commit messages to show (per Git repo)'),
      '#description' => $this->t('A value of zero (0) will not show any commit messages. Default: <em>2</em>'),
      '#default_value' => $config->get('number_commit_messages'),
      '#min' => 0,
      '#max' => 9,
      '#size' => 1,
      '#maxlength' => 1,
    ];
    $form['fuzzy_timestamp_granularity'] = [
      '#type' => 'number',
      '#title' => $this->t('Fuzzy timestamp granularity (<em>time ago</em> date format)'),
      '#description' => $this->t("Examples of output with the current granularity: <em><strong>@sample_old ago</strong></em> (years back), <em><strong>@sample_recent ago</strong></em> (recent)<br />The minimum of '1' results for example in <em>'2 weeks ago'</em>. A value of '5' (or higher) in <em>'2 weeks 4 days 18 hours 10 min 15 sec ago'</em>.<br />A low granularity is understood at a glance though less accurate. The exact date and time is shown on mouse-over anyway (see below). Default: <em>1</em><br />Granularity is not the same as the number of units. Following units in the hierarchy will not get skipped. For example <em>'5 months 19 hours'</em> will always be shown as <em>'5 months'</em>, independent from the granularity set. After a week it will then turn into <em>'5 months 1 week'</em>.", [
        '@sample_old' => \Drupal::service("date.formatter")
          ->formatInterval(81950198, $config->get('fuzzy_timestamp_granularity')), '@sample_recent' => \Drupal::service("date.formatter")
          ->formatInterval(76477, $config->get('fuzzy_timestamp_granularity')),
      ]),
      '#default_value' => $config->get('fuzzy_timestamp_granularity'),
      '#min' => 1,
      '#max' => 9,
      '#size' => 1,
      '#maxlength' => 1,
    ];
    $php_manual = Link::fromTextAndUrl($this->t('PHP manual'), Url::fromUri('https://www.php.net/manual/en/function.date.php'));
    $php_manual = $php_manual->toRenderable();
    $php_manual['#attributes'] = ['title' => $this->t('PHP: date - Manual'), 'target' => '_blank'];
    $strtotime = Link::fromTextAndUrl($this->t('strtotime'), Url::fromUri('https://www.php.net/manual/en/function.strtotime.php'));
    $strtotime = $strtotime->toRenderable();
    $strtotime['#attributes'] = ['title' => $this->t('PHP: strtotime - Manual'), 'target' => '_blank'];
    $form['exact_date_format'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Exact date and time format shown as tooltip on the fuzzy timestamp'),
      '#description' => $this->t("Example output of the current time: <em><strong>@sample</strong></em><br />A user-defined date format. See the @php_manual for available options. Leave empty to reset to the default: <em>Y-m-d H:i:s O</em><br />Validated by passing the escaped format string through the PHP function @strtotime to ensure we get a sensible date.", [
        '@sample' => date($config->get('exact_date_format')),
        '@php_manual' => render($php_manual),
        '@strtotime' => render($strtotime),
      ]),
      '#default_value' => $config->get('exact_date_format'),
      '#size' => 30,
      '#maxlength' => 60,
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $extras = array_map('trim', explode(PHP_EOL, $form_state->getValue('extra_folders')));
    foreach ($extras as $extra) {
      $exists = shell_exec('[ -d ' . escapeshellcmd('/' . trim($extra, " /\t\n\r")) . ' ] && echo "1"');
      if (!$exists) {
        $form_state->setErrorByName('extra_folders', $this->t('Directory %directory does not exist.', ['%directory' => '/' . trim($extra, " /\t\n\r")]));
      }
    }
    $aggregates = array_map('trim', explode(PHP_EOL, $form_state->getValue('aggregate_folders')));
    foreach ($aggregates as $aggregate) {
      if (!preg_match('/^[\w|\/]+$/', $aggregate) && !empty(trim($form_state->getValue('aggregate_folders')))) {
        $form_state->setErrorByName('aggregate_folders', $this->t('Directory %directory is invalid.', ['%directory' => trim($aggregate, " /\t\n\r")]));
      }
    }
    $valid_date = (bool) strtotime(date(preg_replace('/\\\\\w+/', '', $form_state->getValue('exact_date_format'))));
    if (!$valid_date && !empty($form_state->getValue('exact_date_format'))) {
      $form_state->setErrorByName('exact_date_format', $this->t('Invalid format.'));
    }
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $extra_folders = [];
    $extras = array_filter(explode(PHP_EOL, trim($form_state->getValue('extra_folders'))));
    foreach ($extras as $extra) {
      $extra_folders[] = '/' . trim($extra, " /\t\n\r");
    }
    $extra_folders = implode(PHP_EOL, $extra_folders);
    $aggregate_folders = [];
    $aggregates = empty(trim($form_state->getValue('aggregate_folders'))) ? "vendor\r\nweb/modules/contrib\r\ndocroot/modules/contrib" : $form_state->getValue('aggregate_folders');
    $aggregates = explode(PHP_EOL, $aggregates);
    foreach ($aggregates as $aggregate) {
      $aggregate_folders[] = trim($aggregate, " /\t\n\r");
    }
    $aggregate_folders = implode(PHP_EOL, $aggregate_folders);
    $number_commit_messages = $form_state->getValue('number_commit_messages');
    $number_commit_messages = empty($number_commit_messages) && (string) $number_commit_messages !== '0' ? 2 : $number_commit_messages;
    $fuzzy_timestamp_granularity = $form_state->getValue('fuzzy_timestamp_granularity');
    $fuzzy_timestamp_granularity = empty($fuzzy_timestamp_granularity) ? 1 : $fuzzy_timestamp_granularity;
    $exact_date_format = trim($form_state->getValue('exact_date_format'));
    $exact_date_format = empty($exact_date_format) ? 'Y-m-d H:i:s O' : $exact_date_format;
    $this->config('gitinfo.settings')
      ->set('connect', $form_state->getValue('connect'))
      ->set('extra_folders', $extra_folders)
      ->set('aggregate_folders', $aggregate_folders)
      ->set('number_commit_messages', $number_commit_messages)
      ->set('fuzzy_timestamp_granularity', $fuzzy_timestamp_granularity)
      ->set('exact_date_format', $exact_date_format)
      ->save();
    parent::submitForm($form, $form_state);
  }

}
